#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>

#define SEG7_ZERO ((1 << PB6) | (1 << PB5) | (1 << PB1) | (1 << PB2) | (1 << PB3) | (1 << PB7))
#define SEG7_UNU ((1 << PB1) | (1 << PB7))
#define SEG7_DOI ((1 << PB7) | (1 << PB6) | (1 << PB4) | (1 << PB3) | (1 << PB2))
#define SEG7_TREI ((1 << PB7) | (1 << PB6) | (1 << PB4) | (1 << PB2) | (1 << PB1))
#define SEG7_PATRU ((1 << PB1) | (1 << PB7) | (1 << PB5) | (1 << PB4))
#define SEG7_CINCI ((1 << PB6) | (1 << PB5) | (1 << PB4) | (1 << PB1) | (1 << PB2))
#define SEG7_SASE ((1 << PB6) | (1 << PB5) | (1 << PB4) | (1 << PB1) | (1 << PB2) | (1 << PB3))
#define SEG7_SAPTE ((1 << PB1) | (1 << PB7) | (1 << PB6))
#define SEG7_OPT ((1 << PB6) | (1 << PB5) | (1 << PB4) | (1 << PB1) | (1 << PB2) | (1 << PB3) | (1 << PB7))
#define SEG7_NOUA ((1 << PB6) | (1 << PB5) | (1 << PB4) | (1 << PB1) | (1 << PB2) | (1 << PB7))

#define ADC_AREF_VOLTAGE 3.3
#define NUM_NOTE		8
#define NUM_OCTAVE		8
#define OCTAVA_INITIALA 4
#define ADC_THRESHOLD 650

int frecventa_nota[NUM_OCTAVE][NUM_NOTE] = {
	{16, 18, 21, 22, 25, 28, 31, 33},
	{33, 37, 41, 44, 49, 55, 62, 65},
	{65, 73, 82, 87, 98, 110, 123, 131},
	{131, 147, 165, 175, 196, 220, 247, 262},
	{262, 294, 330, 349, 392, 440, 494, 523},
	{523, 587, 659, 698, 784, 880, 988, 1046},
	{1046, 1175, 1319, 1397, 1568, 1760, 1976, 2093},
	{2093, 2349, 2637, 2794, 3136, 3520, 3951, 4186}
};

int octava_curenta;

void ADC_init(void)
{
    // enable ADC with:
    // * reference AVCC with external capacitor at AREF pin
    // * without left adjust of conversion result
    // * no auto-trigger
    // * no interrupt
    // * prescaler at 32
    ADMUX = (1 << REFS0);
    ADCSRA = (1 << ADEN) | (5 << ADPS0);
}

uint16_t ADC_get(uint8_t channel)
{
    // start ADC conversion on "channel"
    // wait for completion
    // return the result
    ADMUX = (ADMUX & ~(0x1f << MUX0)) | channel;

    ADCSRA |= (1 << ADSC);
    while(ADCSRA & (1 << ADSC));

    return ADC;
}

/* Genereaza o nota folosind speaker-ul. */
void speaker_reda(int nota) {
	int perioada_nota = 1000000l / frecventa_nota[octava_curenta][nota];
	int i;

	/*
	 * Nu putem folosi variable ca argument pentru functia
	 * _delay_us(), asa ca folosim acest "workaround".
	 */
	PORTD &= ~(1 << PD4);
	for (i = 0; i < perioada_nota / 2; ++i)
		_delay_us(1);

	PORTD |= (1 << PD4);
	for (i = 0; i < perioada_nota / 2; ++i)
		_delay_us(1);
}

/* afiseaza cifra pe display-ul cu 7 segmente */
void aprinde_7seg(int cif) {
	/* reseteaza PORTB */
	PORTB = 0x00;
	
	switch (cif) {
		case 0:
			PORTB |= SEG7_ZERO;
			break;
		case 1:
			PORTB |= SEG7_UNU;
			break;
		case 2:
			PORTB |= SEG7_DOI;
			break;
		case 3:
			PORTB |= SEG7_TREI;
			break;
		case 4:
			PORTB |= SEG7_PATRU;
			break;
		case 5:
			PORTB |= SEG7_CINCI;
			break;
		case 6:
			PORTB |= SEG7_SASE;
			break;
		case 7:
			PORTB |= SEG7_SAPTE;
			break;
		case 8:
			PORTB |= SEG7_OPT;
			break;
		case 9:
			PORTB |= SEG7_NOUA;
			break;
		default:
			PORTB = 0x00;
	}
}

ISR(PCINT2_vect)
{
    if ((PINC & (1 << PC0)) == 0)
    {
		if (octava_curenta > 0) {
			octava_curenta--;
			aprinde_7seg(octava_curenta);
		}
    } else if((PINC & (1 << PC1)) == 0)
	{
		if (octava_curenta < NUM_OCTAVE - 1) {
			octava_curenta++;
			aprinde_7seg(octava_curenta);
		}
	}
}

int main(void) {
	
	/* setez portul B ca iesire */
	DDRB = 0xFF;
	/* seteaza octava initiala ca fiind 4 */
	octava_curenta = OCTAVA_INITIALA;
	aprinde_7seg(octava_curenta);
	
	DDRD |= (1 << PD7);
	
	/* Setez pinul PD4 ca iesire pentru buzzer. */
	DDRD |= (1 << PD4);

	/* Opresc buzzer-ul. */
	PORTD |= (1 << PD4);
	
	/* setez portul C ca si intrare si seteaza rezistenta de pull-up*/
	DDRC &= !((1 << PC0) | (1 << PC1));
	PORTC |= (1 << PC0);
	PORTC |= (1 << PC1);
	
	int nota_curenta = -1;
	
	/* initializare convertor analog-digital */
	ADC_init();
	
	/* activare intreruperi */
	sei();
	// Activez PIN CHANGE INTERRUPT ENABLE pentru PCINT23:16 (portul C)
    PCICR |= (1 << PCIE2);
	// Activez intreruperea pe butoanele PC0 si PC1
    PCMSK2 |= (1 << PCINT16);
    PCMSK2 |= (1 << PCINT17);
	
	/* verifica daca o coarda a fost atinsa */
	while (1) {
		if (ADC_get(0) < ADC_THRESHOLD)
			nota_curenta = 0;
		else if (ADC_get(1) < ADC_THRESHOLD)
			nota_curenta = 1;
		else if (ADC_get(2) < ADC_THRESHOLD)
			nota_curenta = 2;
		else if (ADC_get(3) < ADC_THRESHOLD)
			nota_curenta = 3;
		else if (ADC_get(4) < ADC_THRESHOLD)
			nota_curenta = 4;
		else if (ADC_get(5) < ADC_THRESHOLD)
			nota_curenta = 5;
		else if (ADC_get(6) < ADC_THRESHOLD)
			nota_curenta = 6;
		else if (ADC_get(7) < ADC_THRESHOLD)
			nota_curenta = 7;
		else
			nota_curenta = -1;
		
		if (nota_curenta != -1) {
			/* Reda nota. */
			speaker_reda(nota_curenta);
		}
	}
	
	return 0;
}